"""
	A huge map of every myGengo API endpoint to a function definition in mygengo-python.

	Parameters that need to be embedded in the URL are treated with mustaches, e.g:

	{{bert}}, etc

	When creating new endpoint definitions, keep in mind that the name of the mustache
	will be replaced with the keyword that gets passed in to the function at call time.

	i.e, in this case, if I pass bert = 47 to any function, {{bert}} will be replaced
	with 47, instead of defaulting to 1 (said defaulting takes place at conversion time).
"""

# myGengo API urls. {{version}} gets replaced with v1/etc
# at run time.
api_urls = {
	'sandbox': 'http://api.sandbox.mygengo.com/{{version}}',
	'base': 'http://api.mygengo.com/{{version}}'
}

# The API endpoint 'table', 'database', 'hash', 'dictionary', whatever
# you'd like to call it. To keep things uber nice and organized, we secure away
# all the endpoints here with easily replaceable scenarios. Win!
apihash  = {
	# All Account-information based methods...
	'getAccountStats': {
		'url': '/account/stats',
		'method': 'GET',
	},
	'getAccountBalance': {
		'url': '/account/balance',
		'method': 'GET',
	},

	# Creating new translation requests.
	'postTranslationJob': {
		'url': '/translate/job',
		'method': 'POST',
	},
	'postTranslationJobs': {
		'url': '/translate/jobs',
		'method': 'POST',
	},

	# Updating an existing translation request.
	'updateTranslationJob': {
		'url': '/translate/job/{{id}}',
		'method': 'PUT',
	},
	'updateTranslationJobs': {
		'url': '/translate/jobs',
		'method': 'PUT',
	},

	# Viewing existing translation requests.
	'getTranslationJob': {
		'url': '/translate/job/{{id}}',
		'method': 'GET',
	},
	'getTranslationJobs': {
		'url': '/translate/jobs{{ids}}',
		'method': 'GET',
	},
	'getTranslationJobBatch': {
		'url': '/translate/jobs/group/{{group_id}}',
		'method': 'GET',
	},

	# Get a quote for how much a given job will cost.
	'determineTranslationCost': {
		'url': '/translate/service/quote',
		'method': 'POST',
	},

	# Deal with comments and other metadata about a TranslationJob in progress.
	'postTranslationJobComment': {
		'url': '/translate/job/{{id}}/comment',
		'method': 'POST',
	},
	'getTranslationJobComments': {
		'url': '/translate/job/{{id}}/comments',
		'method': 'GET',
	},
	'getTranslationJobFeedback': {
		'url': '/translate/job/{{id}}/feedback',
		'method': 'GET',
	},
	'getTranslationJobRevisions': {
		'url': '/translate/job/{{id}}/revisions',
		'method': 'GET',
	},
	'getTranslationJobRevision': {
		'url': '/translate/job/{{id}}/revisions/{{revision_id}}',
		'method': 'GET',
	},
	'getTranslationJobPreviewImage': {
		'url': '/translate/job/{{id}}/preview',
		'method': 'GET',
	},

	# Delete a job...
	'deleteTranslationJob': {
		'url': '/translate/job/{{id}}',
		'method': 'DELETE',
	},
	'deleteTranslationJobs': {
		'url': '/translate/jobs',
		'method': 'DELETE',
	},

	# Translation Service language information. Holds information
	# about which languages can be converted to which, etc.
	'getServiceLanguagePairs': {
		'url': '/translate/service/language_pairs',
		'method': 'GET',
	},
	'getServiceLanguages': {
		'url': '/translate/service/languages',
		'method': 'GET',
	},
}



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
	A set of tests for myGengo. They all require internet connections.
	In fact, this entire library and API requires an internet connection.
	Don't complain.

	Also keep in mind that all the test cases test against the Sandbox API,
	since... well, it makes more sense to do that. If you, for some odd reason,
	want/need to test against the regular API, modify the SANDBOX flag at the top
	of this file.

	-- Ryan McGrath (ryan @ venodesigns dat net)
"""

import unittest
import random

from pprint import pprint
from mygengo import MyGengo, MyGengoError, MyGengoAuthError

# test_keys is a file I use to store my keys separately. It's stuck in the
# .gitignore as well, so feel free to use it for your own purposes, but there's
# no warranty with this. Keys are needed to run the tests below, though, so if want to
# run the tests, copy over the example keys file and throw your information in.
#
# e.g, cp test_keys_example.py test_keys.py
from test_keys import public_key, private_key

# We test in the myGengo sandbox for all these tests. Flip this if you need/want to.
SANDBOX = True

class TestMyGengoCore(unittest.TestCase):
	"""
		Handles testing the core parts of myGengo (i.e, authentication signing, etc).
	"""
	def test_MethodDoesNotExist(self):
		myGengo = MyGengo(public_key = public_key, private_key = private_key, sandbox = SANDBOX)
		# With how we do functions, AttributeError is a bit tricky to catch...
		self.assertRaises(AttributeError, getattr, myGengo, 'bert')
	
	def test_MyGengoAuthNoCredentials(self):
		myGengo = MyGengo(public_key = '', private_key = '', sandbox = SANDBOX)
		self.assertRaises(MyGengoError, myGengo.getAccountStats)
	
	def test_MyGengoAuthBadCredentials(self):
		myGengo = MyGengo(public_key = 'bert', private_key = 'beeeerrrttttt', sandbox = SANDBOX)
		self.assertRaises(MyGengoAuthError, myGengo.getAccountStats)


class TestAccountMethods(unittest.TestCase):
	"""
		Tests the methods that deal with retrieving basic information about
		the account you're authenticating as. Checks for one property on each
		method; if your keys work with these methods, well...
	"""
	def test_getAccountStats(self):
		myGengo = MyGengo(public_key = public_key, private_key = private_key, sandbox = SANDBOX)
		stats = myGengo.getAccountStats()
		self.assertEqual(stats['opstat'], 'ok')
	
	def test_getAccountBalance(self):
		myGengo = MyGengo(public_key = public_key, private_key = private_key, sandbox = SANDBOX)
		balance = myGengo.getAccountBalance()
		self.assertEqual(balance['opstat'], 'ok')


class TestLanguageServiceMethods(unittest.TestCase):
	"""
		Tests the methods that deal with getting information about language-translation
		service support from myGengo.
	"""
	def test_getServiceLanguagePairs(self):
		myGengo = MyGengo(public_key = public_key, private_key = private_key, sandbox = SANDBOX)
		resp = myGengo.getServiceLanguagePairs()
		self.assertEqual(resp['opstat'], 'ok')
	
	def test_getServiceLanguages(self):
		myGengo = MyGengo(public_key = public_key, private_key = private_key, sandbox = SANDBOX)
		resp = myGengo.getServiceLanguages()
		self.assertEqual(resp['opstat'], 'ok')


class TestTranslationJobFlow(unittest.TestCase):
	"""
		Tests the flow of creating a job, updating it, getting the details, and then 
		deleting the job. This is the thick of it!
		
		Flow is as follows:
		
			1: Create a mock job and get an estimate for it (setUp)
			2: Create three jobs - 1 single, 2 batched
			3: Update the first job with some arbitrary information or something
			4: Post a comment on the first job
			6: Perform a hell of a lot of GETs to the myGengo API to check stuff
			7: Delete the job if all went well (teardown phase)
	"""
	def setUp(self):
		"""
			Creates the initial batch of jobs for the other test functions here to operate on.
		"""
		# First we'll create three jobs - one regular, and two at the same time...
		self.myGengo = MyGengo(public_key = public_key, private_key = private_key, sandbox = SANDBOX)		
		self.created_job_ids = []
		
		single_job = {
			'type': 'text',
			'slug': 'Single :: English to Japanese',
			'body_src': 'Test%ding myGe%dngo A%dPI li%dbrary calls.' % (int(random.randrange(1,226,1)), int(random.randrange(1,226,1)), int(random.randrange(1,226,1)), int(random.randrange(1,226,1))),
			'lc_src': 'en',
			'lc_tgt': 'ja',
			'tier': 'standard',
			'auto_approve': 0,
		}
		
		multiple_jobs = {
			'job_1': {
				'type': 'text',
				'slug': 'Multiple :: English to Japanese',
				'body_src': 'H%dow i%ds th%de weather?' % (int(random.randrange(1,226,1)), int(random.randrange(1,226,1)), int(random.randrange(1,226,1))),
				'lc_src': 'en',
				'lc_tgt': 'ja',
				'tier': 'standard',
			},
			'job_2': {
				'type': 'text',
				'slug': 'Multiple :: Japanese To English',
				'body_src': '天%d気%dはど%dうですか' % (int(random.randrange(1,226,1)), int(random.randrange(1,226,1)), int(random.randrange(1,226,1))),
				'lc_src': 'ja',
				'lc_tgt': 'en',
				'tier': 'ultra',
			},
		}
		
		# Now that we've got the job, let's go ahead and see how much it'll cost.
		cost_assessment = self.myGengo.determineTranslationCost(jobs = {'jobs': multiple_jobs})
		self.assertEqual(cost_assessment['opstat'], 'ok')
		
		# If that method worked, sweet. Move on and create three jobs, store their IDs. Make sure we got an ID
		# back, since these methods are otherwise largely useless without that returned data. These tests walk a fine
		# line between testing myGengo and the myGengo API functionality as a whole - watch yourself if you add to this. :)
		job = self.myGengo.postTranslationJob(job = single_job)
		self.assertEqual(job['opstat'], 'ok')
		self.assertIsNotNone(job['response']['job']['job_id'])
		self.created_job_ids.append(job['response']['job']['job_id'])
		
		jobs = self.myGengo.postTranslationJobs(jobs = {'jobs': multiple_jobs})
		self.assertEqual(job['opstat'], 'ok')
		
		# This is a fairly ugly way to check for and retrieve job IDs; in an ideal system you know the keys, and... well,
		# truthfully we do here too. I suppose this is moreso here as an example of how to get IDs in a situation where you
		# don't know the keys. May or may not be useful to some.
		for job_obj in jobs['response']['jobs']:
			for job in job_obj:
				self.assertIsNotNone(job_obj[job]['job_id'])
				self.created_job_ids.append(job_obj[job]['job_id'])
	
	@unittest.skip("We don't test myGengo.getTranslationJobPreviewImage() because it's potentially resource heavy on the myGengo API.")
	def test_getTranslationJobPreviewImage(self):
		"""
			This test could be a bit more granular, but I'm undecided at the moment - testing the response stream
			of this method is more of a Unit Test for myGengo than myGengo. Someone can extend if they see fit, but I 
			currently see no reason to mess with this further.
		"""
		img = self.myGengo.getTranslationJobPreviewImage(id = self.created_job_ids[0])
		self.assertIsNotNone(img)
	
	def test_postJobDataMethods(self):
		"""
			Tests all the myGengo methods that deal with updating jobs, posting comments, etc. test_getJobDataMethods() checks things,
			but they need to exist first - think of this as the alcoholic mother to _getJobDataMethods().
		"""
		# The 'update' method can't really be tested, as it requires the translator having actually done something before
		# it's of any use. Thing is, in automated testing, we don't really have a method to flip the switch on the myGengo end. If we
		# WERE to test this method, it'd look a little something like this:
		#	
		#	updated_job = self.myGengo.updateTranslationJob(id = self.created_job_ids[0], action = {
		#		'action': 'purchase',
		#	})
		#	self.assertEqual(updated_job['opstat'], 'ok')
		
		posted_comment = self.myGengo.postTranslationJobComment(id = self.created_job_ids[0], comment = {
			'body': 'I love lamp oh mai gawd',
		})
		self.assertEqual(posted_comment['opstat'], 'ok')
	
	def test_getJobDataMethods(self):
		"""
			Test a ton of methods that GET data from the myGengo API, based on the jobs we've created and such.
			
			These are separate from the other GET request methods because this might be a huge nuisance to their API,
			and I figure it's worth separating out the pain-point test cases so they could be disabled easily in a 
			distribution or something.
		"""
		# Pull down data about one specific job...
		job = self.myGengo.getTranslationJob(id = self.created_job_ids[0])
		self.assertEqual(job['opstat'], 'ok')
		
		# Pull down the 10 most recently submitted jobs.
		jobs = self.myGengo.getTranslationJobs()
		self.assertEqual(jobs['opstat'], 'ok')
		
		# Test getting the batch that a job is in...
		job_batch = self.myGengo.getTranslationJobBatch(id = self.created_job_ids[1])
		self.assertEqual(job_batch['opstat'], 'ok')
		
		# Pull down the comment(s) we created earlier in this test suite.
		job_comments = self.myGengo.getTranslationJobComments(id = self.created_job_ids[0])
		self.assertEqual(job_comments['opstat'], 'ok')
		
		# Pull down feedback. This should work fine, but there'll be no feedback or anything, so... meh.
		feedback = self.myGengo.getTranslationJobFeedback(id = self.created_job_ids[0])
		self.assertEqual(feedback['opstat'], 'ok')
		
		# Lastly, pull down any revisions that definitely didn't occur due to this being a simulated test.
		revisions = self.myGengo.getTranslationJobRevisions(id = self.created_job_ids[0])
		self.assertEqual(revisions['opstat'], 'ok')
		
		# So it's worth noting here that we can't really test getTranslationJobRevision(), because no real revisions
		# exist at this point, and a revision ID is required to pull that method off successfully. Bai now.
	
	def tearDown(self):
		"""
			Delete every job we've created for this somewhat ridiculously thorough testing scenario.
		"""
		for id in self.created_job_ids:
			deleted_job = self.myGengo.deleteTranslationJob(id = id)
			self.assertEqual(deleted_job['opstat'], 'ok')


if __name__ == '__main__':
	unittest.main()



#!/usr/bin/python

"""
	This file mainly exists for storing your public/private
	keypair in a location that's not tracked through Git. To
	run the myGengo test suite, fill in the values below and copy
	this file over to mygengo_test_keys.py.

	e.g: cp mygengo_test_keys_example.py mygengo_test_keys.py
"""

public_key = ''
private_key = ''



#!/usr/bin/python

"""
	This file mainly exists for storing your public/private
	keypair in a location that's not tracked through Git. To
	run the pyGengo test suite, fill in the values below and copy
	this file over to pygengo_test_keys.py.

	e.g: cp pygengo_test_keys_example.py pygengo_test_keys.py
"""
#public_key='w5|ABkLj]DOko)3(B_Tw1ipILO_#l04d}zhHJaN3=jIPU^mlxvmpZN(@B_pIs~LB'
#private_key='gNtQ4k4JEu$zPY8qQ0zTvdV-RAf47lTGVblwo~wt@wiuDK[$qZ=DJ-4PZZ$nsEYs'
public_key='~8IuuuEQmqm3OM}vy5PSXzMXTUg=uezN^gT]}zoP_Fd7D#8KE#ceo#)PutT|9qZ1'
private_key='X[5pmR[I0pMZy(C]SrllsEaWm8Vd^kq-s1uvsm-|GPIJHhJ64cu#q~nk1XWT3F@2'



#!/usr/bin/python

"""
	Official Python library for interfacing with the myGengo API.
	It's released under a BSD-style license and totally, freely available. Check it out on Github
	if you find any issues!

	Questions, comments? ryan@mygengo.com
"""

__author__ = 'Ryan McGrath <ryan@mygengo.com>'
__version__ = '1.3.2'

import httplib2, mimetypes, mimetools, re, hmac

from pprint import pprint
from hashlib import sha1
from urllib import urlencode, quote
from time import time
from operator import itemgetter

# mockdb is a file with a dictionary of every API endpoint for myGengo.
from mockdb import api_urls, apihash

# There are some special setups (like, oh, a Django application) where
# simplejson exists behind the scenes anyway. Past Python 2.6, this should
# never really cause any problems to begin with.
try:
	# Python 2.6 and up
	import json
except ImportError:
	try:
		# Python 2.6 and below (2.4/2.5, 2.3 is not guranteed to work with this library to begin with)
		import simplejson as json
	except ImportError:
		try:
			# This case gets rarer by the day, but if we need to, we can pull it from Django provided it's there.
			from django.utils import simplejson as json
		except:
			# Seriously wtf is wrong with you if you get this Exception.
			raise Exception("mygengo requires the simplejson library (or Python 2.6+) to work. http://www.undefined.org/python/")

class MyGengoError(Exception):
	"""
		Generic error class, catch-all for most MyGengo issues.
		Special cases are handled by APILimit and AuthError.

		Note: You need to explicitly import them into your code, e.g:

		from mygengo import MyGengoError, MyGengoAuthError
	"""
	def __init__(self, msg, error_code=None):
		self.msg = msg
		if error_code == 1000:
			# Auth errors tend to be the most requested for their own
			# Exception instances, so give it to the masses, yo.
			raise MyGengoAuthError(msg)
	
	def __str__(self):
		return repr(self.msg)

class MyGengoAuthError(MyGengoError):
	"""
		Raised when you try to access a protected resource and it fails due to some issue with
		your authentication.
	"""
	def __init__(self, msg):
		self.msg = msg
	
	def __str__(self):
		return repr(self.msg)

class MyGengo(object):
	def __init__(self, public_key = None, private_key = None, sandbox = False, api_version = '1.1', headers = None, debug = False, hackathon = False):
		"""MyGengo(public_key = None, private_key = None, sandbox = False, headers = None)

			Instantiates an instance of MyGengo.
			
			Parameters:
				public_key - Your 'public' key for myGengo. Retrieve this from your account information if you want to do authenticated calls.
				private_key - Your 'private' key for myGengo. Retrieve this from your account information if you want to do authenticated calls.
				sandbox - Whether to use the myGengo sandbox or not. Check with myGengo for the differences with this as it may change.
				headers - User agent header, dictionary style ala {'User-Agent': 'Bert'}
				debug - a flag (True/False) which will cause things to properly blow the hell up on exceptions. Useful for debugging. ;P
		"""
		self.api_url = api_urls['sandbox'] if sandbox is True else api_urls['base']	
		self.api_version = str(api_version)
		self.public_key = public_key
		self.private_key = private_key
		# If there's headers, set them, otherwise be an embarassing parent for their own good.
		self.headers = headers
		if self.headers is None:
			self.headers = {'User-agent': 'myGengo Python Library v%s' % __version__}
		# No matter whether we get some supplied or use the generic, tell it we want JSON. ;P
		self.headers['Accept'] = 'application/json'
		self.debug = debug
		self.client = httplib2.Http()
	
	def __getattr__(self, api_call):
		"""
			The most magically awesome block of code you'll ever see.

			Rather than list out 9 million methods for this API, we just keep a table (see above) of
			every API endpoint and their corresponding function id for this library. This pretty much gives
			unlimited flexibility in API support - there's a slight chance of a performance hit here, but if this is
			going to be your bottleneck... well, don't use Python. ;P

			For those who don't get what's going on here, Python classes have this great feature known as __getattr__().
			It's called when an attribute that was called on an object doesn't seem to exist - since it doesn't exist,
			we can take over and find the API method in our table. We then return a function that downloads and parses
			what we're looking for, based on the key/values passed in.

			I'll hate myself for saying this, but this is heavily inspired by Ruby's "method_missing".

			Note: I'm largely borrowing this technique from another API library/wrapper I've written in the past (Twython).
			If you happen to read both sources and find the same text... well, that's why. ;)
		"""
		def get(self, **kwargs):
			# Grab the (hopefully) existing method 'definition' to fire off from our api hash table.
			fn = apihash[api_call]
			
			# Do a check here for specific job sets - we need to support posting multiple jobs
			# at once, so see if there's an dictionary of jobs passed in, pop it out, let things go on as normal,
			# then pick this chain back up below...
			post_data = {}
			if 'job' in kwargs: post_data['job'] = {'job': kwargs.pop('job')}
			if 'jobs' in kwargs: post_data['jobs'] = kwargs.pop('jobs')
			if 'comment' in kwargs: post_data['comment'] = kwargs.pop('comment')
			if 'action' in kwargs: post_data['action'] = kwargs.pop('action')
			if 'job_ids' in kwargs: post_data['job_ids'] = kwargs.pop('job_ids')
			
			# This is for the one-off GET call that acts like a POST call... don't ask. ;P
			if 'ids' in kwargs:
				kwargs['ids'] = '/%s' % ','.join(map(lambda i: str(i), ids))
			
			# Set up a true base URL, abstracting away the need to care about the sandbox mode
			# or API versioning at this stage.
			base_url = self.api_url.replace('{{version}}', 'v%s' % self.api_version)
			
			# Go through and replace any mustaches that are in our API url with their appropriate key/value pairs...
			# NOTE: We pop() here because we don't want the extra data included and messing up our hash down the road.
			base = re.sub(
				'\{\{(?P<m>[a-zA-Z]+)\}\}',
				lambda m: "%s" % kwargs.pop(m.group(1), 'no_argument_specified'), # In case of debugging needs
				base_url + fn['url']
			)
			
			# Build up a proper 'authenticated' url...
			#
			# Note: for further information on what's going on here, it's best to familiarize yourself
			# with the myGengo authentication API. (http://mygengo.com/services/api/dev-docs/authentication)
			query_params = dict([k, quote(v.encode('utf-8'))] for k, v in kwargs.items())
			if self.public_key is not None:
				query_params['api_key'] = self.public_key
			query_params['ts'] = str(int(time()))
			
			# If any further APIs require their own special signing needs, fork here...
			if self.api_version == '1':
				resp, content = self.signAndRequestAPIV1(fn, base, query_params, post_data)
			else:
				resp, content = self.signAndRequestAPILatest(fn, base, query_params, post_data)
			results = json.loads(content)
			
			# See if we got any weird or odd errors back that we can cleanly raise on or something...
			if 'opstat' in results and results['opstat'] != 'ok':
				raise MyGengoError(results['err']['msg'], results['err']['code'])
			
			# If not, screw it, return the junks!
			return results
			
		if api_call in apihash:
			return get.__get__(self)
		else:
			raise AttributeError
	
	def signAndRequestAPIV1(self, fn, base, query_params, post_data = {}):
		"""
			Request signatures between API v1 and later versions of the API differ greatly in
			how they're done, so they're kept in separate methods for now.

			fn - object mapping from mockdb describing POST, etc.
			base - Base URL to ping.
			query_params - Dictionary of data eventually getting sent over to myGengo.
			post_data - Any extra special post data to get sent over.
		"""
		# Encoding jobs becomes a bit different than any other method call, so we catch them and do a little
		# JSON-dumping action. Catching them also allows us to provide some sense of portability between the various
		# job-posting methods in that they can all safely rely on passing dictionaries around. Huzzah!
		if fn['method'] == 'POST' or fn['method'] == 'PUT':
			if 'job' in post_data:
				query_params['data'] = json.dumps(post_data['job'], separators = (',', ':'))
			elif 'jobs' in post_data:
				query_params['data'] = json.dumps(post_data['jobs'], separators = (',', ':'))
			elif 'comment' in post_data:
				query_params['data'] = json.dumps(post_data['comment'], separators = (',', ':'))
			elif 'action' in post_data:
				query_params['data'] = json.dumps(post_data['action'], separators = (',', ':'))
			
			query_json = json.dumps(query_params, separators = (',', ':'), sort_keys = True).replace('/', '\\/')
			query_hmac = hmac.new(self.private_key, query_json, sha1)
			query_params['api_sig'] = query_hmac.hexdigest()
			query_data = urlencode(query_params)
			
			# Httplib2 doesn't assume this for POST requests, but in the case of the myGengo API it's a bit of a hidden necessity.
			headers = self.headers
			headers['Content-Type'] = 'application/x-www-form-urlencoded'
			
			if self.debug is True: print query_data
			return self.client.request(base, fn['method'], headers = headers, body = query_data)
		else:
			query_string = urlencode(sorted(query_params.items(), key = itemgetter(0)))
			if self.private_key is not None:
				query_hmac = hmac.new(self.private_key, query_string, sha1)
				query_params['api_sig'] = query_hmac.hexdigest()
				query_string = urlencode(query_params)
			
			if self.debug is True: print query_string
			return self.client.request(base + '?%s' % query_string, fn['method'], headers = self.headers)

	def signAndRequestAPILatest(self, fn, base, query_params, post_data = {}):
		"""
			Request signatures between API v1 and later versions of the API differ greatly in
			how they're done, so they're kept in separate methods for now.

			This method signs the request with just the timestamp and private key, which is what
			api v1.1 relies on.

			fn - object mapping from mockdb describing POST, etc.
			base - Base URL to ping.
			query_params - Dictionary of data eventually getting sent over to myGengo.
			post_data - Any extra special post data to get sent over.
		"""
		# Encoding jobs becomes a bit different than any other method call, so we catch them and do a little
		# JSON-dumping action. Catching them also allows us to provide some sense of portability between the various
		# job-posting methods in that they can all safely rely on passing dictionaries around. Huzzah!
		if fn['method'] == 'POST' or fn['method'] == 'PUT':
			if 'job' in post_data:
				query_params['data'] = json.dumps(post_data['job'], separators = (',', ':'))
			elif 'jobs' in post_data:
				query_params['data'] = json.dumps(post_data['jobs'], separators = (',', ':'))
			elif 'comment' in post_data:
				query_params['data'] = json.dumps(post_data['comment'], separators = (',', ':'))
			elif 'action' in post_data:
				query_params['data'] = json.dumps(post_data['action'], separators = (',', ':'))
			
			query_json = json.dumps(query_params, separators = (',', ':'), sort_keys = True).replace('/', '\\/')
			query_hmac = hmac.new(self.private_key, query_params['ts'], sha1)
			query_params['api_sig'] = query_hmac.hexdigest()
			query_data = urlencode(query_params)
			
			# Httplib2 doesn't assume this for POST requests, but in the case of the myGengo API it's a bit of a hidden necessity.
			headers = self.headers
			headers['Content-Type'] = 'application/x-www-form-urlencoded'
			
			if self.debug is True: print query_data
			return self.client.request(base, fn['method'], headers = headers, body = query_data)
		else:
			query_string = urlencode(sorted(query_params.items(), key = itemgetter(0)))
			if self.private_key is not None:
				query_hmac = hmac.new(self.private_key, query_params['ts'], sha1)
				query_params['api_sig'] = query_hmac.hexdigest()
				query_string = urlencode(query_params)
			
			if self.debug is True: print query_string
			return self.client.request(base + '?%s' % query_string, fn['method'], headers = self.headers)
	
	@staticmethod
	def unicode2utf8(text):
		try:
			if isinstance(text, unicode):
				text = text.encode('utf-8')
		except:
			pass
		return text



#!/usr/bin/python

import sys, os
from setuptools import setup
from setuptools import find_packages

__author__ = 'Ryan McGrath <ryan@mygengo.com>'
__version__ = '1.3.2'

setup(
	# Basic package information.
	name = 'mygengo',
	version = __version__,
	packages = find_packages(),

	# Packaging options.
	include_package_data = True,

	# Package dependencies.
	install_requires = ['simplejson'],

	# Metadata for PyPI.
	author = 'Ryan McGrath',
	author_email = 'ryan@venodesigns.net',
	license = 'LGPL License',
	url = 'http://github.com/ryanmcgrath/mygengo/tree/master',
	keywords = 'mygengo translation language api japanese english',
	description = 'Official Python library for interfacing with the MyGengo API.',
	long_description = open('README.md').read(),
	classifiers = [
		'Development Status :: 4 - Beta',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: MIT License',
		'Topic :: Software Development :: Libraries :: Python Modules',
		'Topic :: Communications :: Chat',
		'Topic :: Internet'
	]
)
